@@ -25,23 +25,23 @@
 
 #include "SDL_stdinc.h"
 
-#if defined(SDL_LOADSO_DISABLED)
-#undef SDL_VIDEO_VULKAN
-#define SDL_VIDEO_VULKAN 0
-#endif
-
 #if SDL_VIDEO_VULKAN
+#if SDL_LOADSO_DISABLED || SDL_LOADSO_DUMMY
+#error You should not be here.
+#endif
 
 #if SDL_VIDEO_DRIVER_ANDROID
 #define VK_USE_PLATFORM_ANDROID_KHR
 #endif
 #if SDL_VIDEO_DRIVER_COCOA
+#define VK_USE_PLATFORM_METAL_EXT
 #define VK_USE_PLATFORM_MACOS_MVK
 #endif
 #if SDL_VIDEO_DRIVER_DIRECTFB
 #define VK_USE_PLATFORM_DIRECTFB_EXT
 #endif
 #if SDL_VIDEO_DRIVER_UIKIT
+#define VK_USE_PLATFORM_METAL_EXT
 #define VK_USE_PLATFORM_IOS_MVK
 #endif
 #if SDL_VIDEO_DRIVER_WAYLAND
