@@ -169,7 +169,7 @@ loop()
 
                 SDL_SetCursor(cursors[current_cursor]);
 
-                switch (cursor_types[current_cursor]) {
+                switch ((int)cursor_types[current_cursor]) {
                     case (SDL_SystemCursor)-1:        SDL_Log("Custom cursor"); break;
                     case SDL_SYSTEM_CURSOR_ARROW:     SDL_Log("Arrow"); break;
                     case SDL_SYSTEM_CURSOR_IBEAM:     SDL_Log("I-beam"); break;
